//  Cells
//  © Darel Rex Finley, 2008



#define  FRAME_STEP_DELAY    .05   //  (no more than twenty frames per second)
#define  SHARE_PORTION     10      //  (each cell shares one tenth of its value with its four neighbors)



#import "Cells.h"



NSBitmapImageRep  *Bitmap=nil, *BitmapCopy=nil ;
unsigned long      Cell[2][256][256], Set=0, FrameNum=0 ;
NSImage           *Image  =nil ;
bool               Running=NO  ;
BYTE              *Planes[5]   ;



@implementation Cells



- (IBAction) handleGoStopButton:(id) sender {
  if (!Running) {
    [      GoButton setTitle   :@"Stop"]; Running=YES;
    [   ResetButton setEnabled :NO     ];
    [StopFrameField setEditable:NO     ]; [self performSelector:@selector(stepFrame) withObject:nil afterDelay:FRAME_STEP_DELAY]; }
  else {
    [      GoButton setTitle   :@"Go"  ]; Running= NO;
    [   ResetButton setEnabled :YES    ];
    [StopFrameField setEditable:YES    ]; }}



- (void) stepFrame {

  long  i, iMinus, iPlus ;
  long  j, jMinus, jPlus ;

  if (Running) {

    //  Cause each cell to share a portion of its value with its closest neighbors.
    for   (j=0; j<256; j++) {
      jMinus  =j-1; if (jMinus<  0) jMinus=255;
      jPlus   =j+1; if (jPlus >255) jPlus =  0;
      for (i=0; i<256; i++) {
        iMinus=i-1; if (iMinus<  0) iMinus=255;
        iPlus =i+1; if (iPlus >255) iPlus =  0;
        Cell[1-Set][i     ][j     ]=
        Cell[  Set][i     ][j     ]-
        Cell[  Set][i     ][j     ]/SHARE_PORTION/4*4+
        Cell[  Set][iMinus][j     ]/SHARE_PORTION/4+
        Cell[  Set][iPlus ][j     ]/SHARE_PORTION/4+
        Cell[  Set][i     ][jMinus]/SHARE_PORTION/4+
        Cell[  Set][i     ][jPlus ]/SHARE_PORTION/4; }}
    Set=1-Set;

    //  Transfer the cell values to the bitmap.
    transferCellValuesToBitmap();

    //  Update the user-viewable image.
    [self recreateImageForImageView];

    //  Update the frame number.
    FrameNum++; [FrameField setStringValue:[NSString stringWithFormat:@"%u",FrameNum]];

    //  Stop the render if the stop-frame has been reached.
    if (FrameNum==[StopFrameField intValue]) {
      [self handleGoStopButton:nil]; return; }

    //  Prime the next frame for execution.
    [self performSelector:@selector(stepFrame) withObject:nil afterDelay:FRAME_STEP_DELAY]; }}



- (IBAction) handleResetButton:(id) sender {

  //  Reset the field values.
  [    FrameField setStringValue:@"0"]; FrameNum=0;
  [StopFrameField setStringValue:@ ""];

  //  Reset the cell values.
  resetCellValues();

  //  Transfer the cell values to the bitmap.
  transferCellValuesToBitmap();

  //  Import the bitmap into the window’s image view.
  [self recreateImageForImageView]; }



- (void) recreateImageForImageView {
  if (Image) {
    [ImageView setImage:nil]; [Image release]; Image=nil; }
  createImage(); [ImageView setImage:Image]; }



//  This method automatically fires on app launch.

- (void) awakeFromNib {

  //  Create the bitmap image structures that will be used the entire time the app is running.
  createBitmap();

  //  Get data planes from the bitmap.
  [Bitmap getBitmapDataPlanes:&Planes[0]];

  //  Reset the app to its starting state.
  [self handleResetButton:nil]; }



- (NSApplicationTerminateReply) applicationShouldTerminate:(NSApplication *) sender {

  //  Release the allocated objects before terminating the app.
  if (Image     ) [Image      release];
  if (Bitmap    ) [Bitmap     release];
  if (BitmapCopy) [BitmapCopy release];

  //  All clear -- allow the app to terminate.
  return NSTerminateNow; }



@end



//  Create the bitmap for the image.

void createBitmap() {

  Bitmap=[NSBitmapImageRep alloc];

  [Bitmap initWithBitmapDataPlanes:nil
  pixelsWide              :256
  pixelsHigh              :256
  bitsPerSample           :  8
  samplesPerPixel         :  1
  hasAlpha                : NO
  isPlanar                : NO
  colorSpaceName          :NSDeviceWhiteColorSpace
  bytesPerRow             :256
  bitsPerPixel            :  8]; }



//  Create an OS image structure for the main, preview image.  ...make absolutely sure there’s no memory leak associated with this

void createImage() {

  //  Create a copy of the bitmap.
  if (BitmapCopy) [BitmapCopy release];
  BitmapCopy=[Bitmap copy];

  //  Create the image, and assign the bitmap copy to it.
  Image=[NSImage new]; [Image addRepresentation:BitmapCopy]; }



void resetCellValues() {

  short  i, j ;

  //  Zero all cells.
  for (i=0; i<256; i++) for (j=0; j<256; j++) Cell[Set][i][j]=0L;

  //  Give one central cell a large value.
  Cell[Set][128][128]=10000000L; }



void transferCellValuesToBitmap() {

  long  i, j, val ;

  for   (j=0L; j<256L; j++) {
    for (i=0L; i<256L; i++) {
      val=Cell[Set][i][j]; if (val>255L) val=255L;
      Planes[0][j*256L+i]=(BYTE) val; }}}